# Zach Gildersleeve
# zgildersleeve@gmail.com
# CS6630 - Scientific Visualization
# Project 4
# November 22, 2006
#
# contour.tcl - This script, originally created for project 3,
# uses a vtkContourFilter to create
# isosurfaces from the mummy dataset.  A GUI allows the
# user to select between the skin contour value, the bone contour
# value, a skin-bone transfer function, and a slider allowing
# custom contour value positioning.

catch {load vtktcl}
set VTK_TCL ".."
set VTK_DATA "../vtkdata"


vtkStructuredPointsReader head
	head SetFileName $VTK_DATA/mummy.128.vtk
	head Update

##########
# Create the skin actor
#
vtkContourFilter contour_skin
	contour_skin SetInput [head GetOutput]
	contour_skin SetNumberOfContours 1
	contour_skin SetValue 0 80

vtkPolyDataNormals normals_skin
	normals_skin SetInput [contour_skin GetOutput]
	normals_skin SetFeatureAngle 60
	normals_skin ConsistencyOff
	normals_skin SplittingOff

vtkPolyDataMapper mapper_skin
	mapper_skin SetInput [normals_skin GetOutput]
	mapper_skin ScalarVisibilityOff

vtkActor actor_skin
	actor_skin SetMapper mapper_skin
	[actor_skin GetProperty] SetColor 0.8 0.2 0.2
	actor_skin RotateX -90

##########
# Create the bone actor
#
vtkContourFilter contour_bone
	contour_bone SetInput [head GetOutput]
	contour_bone SetNumberOfContours 1
	contour_bone SetValue 0 120

vtkPolyDataNormals normals_bone
	normals_bone SetInput [contour_bone GetOutput]
	normals_bone SetFeatureAngle 60
	normals_bone ConsistencyOff
	normals_bone SplittingOff

vtkPolyDataMapper mapper_bone
	mapper_bone SetInput [normals_bone GetOutput]
	mapper_bone ScalarVisibilityOff

vtkActor actor_bone
	actor_bone SetMapper mapper_bone
	[actor_bone GetProperty] SetColor 0.9 0.9 1.0
	[actor_skin GetProperty] SetOpacity 0.3
	actor_bone RotateX -90

##########
# Create the Renderer and Windows
#
vtkRenderer ren1
	
vtkRenderWindow renWin
	renWin AddRenderer ren1
	renWin SetSize 512 512
vtkRenderWindowInteractor iren
	iren SetRenderWindow renWin
	ren1 AddActor actor_skin
	ren1 AddActor actor_bone
	ren1 SetBackground 0.5 0.5 0.5
	ren1 ResetCamera

vtkInteractorStyleTrackballCamera style
	iren SetInteractorStyle style

##########
# Adapted from camframe.tcl
#
proc saveView {} {
  set chan [open "saveView.tcl" w]
  set cam [ren1 GetActiveCamera]
  puts $chan "set cam \[ren1 GetActiveCamera\]"
  puts $chan [format "\$cam SetPosition %s" [$cam GetPosition]]
  puts $chan [format "\$cam SetViewUp %s" [$cam GetViewUp]]
  puts $chan [format "\$cam SetFocalPoint %s" [$cam GetFocalPoint]]
  puts $chan [format "\$cam SetViewAngle %s" [$cam GetViewAngle]]
  puts $chan [format "renWin SetSize %s" [renWin GetSize]]
  puts $chan "ren1 ResetCameraClippingRange"
  close $chan
}

# if a previous run of this program generated the camera information file,
# then read it in now, before the first rendering
if {[file exists "saveView.tcl"]} {
    source "saveView.tcl"
}

renWin Render

#########
# Set up the GUI widgets
#
frame .f1
frame .f1.keepMe
button .f1.keepMe.quit -text "Quit" -command {exit}
button .f1.keepMe.reset -text "Reset Camera" -command resetCamera
button .f1.keepMe.save -text "save view" -command saveView

##########
# Arrange the widgets and GUI elements in each
# respective frame
#
grid .f1.keepMe -row 0 -column 0
grid .f1.keepMe.quit -row 1 -column 0
grid .f1.keepMe.reset -row 2 -column 0
grid .f1.keepMe.save -row 3 -column 0

##########
# Arrange the frames in the GUI window
#
grid .f1 -row 0 -sticky ew

###
# Resets the camera. 
#
proc resetCamera {args} {
	ren1 ResetCamera
	ren1 ResetCameraClippingRange
	[ren1 GetActiveCamera] OrthogonalizeViewUp
	[ren1 GetActiveCamera] SetViewUp 0 1 0
	renWin Render
}