# Zach Gildersleeve
# zgildersleeve@gmail.com
# CS6630 - Scientific Visualization
# Project 4
# November 22, 2006
#
# mip.tcl - This script follows the same pipeline as
# volume.tcl, but uses a vtkVolueRayCastMIPFunction object
# to build a maximum intensity projection image instead
# of a composited image.

catch {load vtktcl}
set VTK_TCL ".."
set VTK_DATA "../vtkdata"

##########
# Read in the data sets
#
vtkStructuredPointsReader reader_mummy
	reader_mummy SetFileName $VTK_DATA/mummy.128.vtk
	reader_mummy Update

##########
# Build the different transfer function components
#
vtkPiecewiseFunction opacityTransferFunction_mummy
	opacityTransferFunction_mummy AddPoint	10	0.0
	opacityTransferFunction_mummy AddPoint	80	0.2
	opacityTransferFunction_mummy AddPoint	115	0.5
	opacityTransferFunction_mummy AddPoint	255	0.0

vtkColorTransferFunction colorTransferFunction_mummy
	colorTransferFunction_mummy AddRGBPoint	      0.0	1.0	1.0	1.0
	colorTransferFunction_mummy AddRGBPoint      60.0	1.0	1.0	1.0
	colorTransferFunction_mummy AddRGBPoint     100.0	1.0	1.0	1.0
	colorTransferFunction_mummy AddRGBPoint     110.0	1.0	1.0	1.0

##########
# Create the vtkVolume via a vtkVolumeRayCastMapper mapping function
#
vtkVolumeProperty volumeProperty
	volumeProperty SetColor colorTransferFunction_mummy
	volumeProperty SetScalarOpacity opacityTransferFunction_mummy
	volumeProperty ShadeOn
	volumeProperty SetSpecular 0.3
	volumeProperty SetInterpolationTypeToLinear

vtkVolumeRayCastMIPFunction MIPFunction

vtkVolumeRayCastMapper volumeMapper
	volumeMapper SetInput [reader_mummy GetOutput]
	volumeMapper SetVolumeRayCastFunction MIPFunction
	volumeMapper SetSampleDistance 0.25

vtkVolume volume
	volume SetMapper volumeMapper
	volume SetProperty volumeProperty
	volume RotateX -90
	volume RotateZ -90

##########
# Create the Renderer and Windows
#
vtkRenderer ren1
	
vtkRenderWindow renWin
	renWin AddRenderer ren1
	renWin SetSize 512 512
vtkRenderWindowInteractor iren
	iren SetRenderWindow renWin
	ren1 AddVolume volume
	ren1 SetBackground 0.0 0.0 0.0
	ren1 ResetCamera

vtkInteractorStyleTrackballCamera style
	iren SetInteractorStyle style

##########
# Adapted from camframe.tcl
#
proc saveView {} {
  set chan [open "saveView.tcl" w]
  set cam [ren1 GetActiveCamera]
  puts $chan "set cam \[ren1 GetActiveCamera\]"
  puts $chan [format "\$cam SetPosition %s" [$cam GetPosition]]
  puts $chan [format "\$cam SetViewUp %s" [$cam GetViewUp]]
  puts $chan [format "\$cam SetFocalPoint %s" [$cam GetFocalPoint]]
  puts $chan [format "\$cam SetViewAngle %s" [$cam GetViewAngle]]
  puts $chan [format "renWin SetSize %s" [renWin GetSize]]
  puts $chan "ren1 ResetCameraClippingRange"
  close $chan
}

# if a previous run of this program generated the camera information file,
# then read it in now, before the first rendering
if {[file exists "saveView.tcl"]} {
    source "saveView.tcl"
}

renWin Render

#########
# Set up the GUI widgets
#
frame .f1
frame .f1.keepMe
button .f1.keepMe.quit -text "Quit" -command {exit}
button .f1.keepMe.reset -text "Reset Camera" -command resetCamera
button .f1.keepMe.save -text "save view" -command saveView

##########
# Arrange the widgets and GUI elements in each
# respective frame
#
grid .f1.keepMe -row 0 -column 0
grid .f1.keepMe.quit -row 1 -column 0
grid .f1.keepMe.reset -row 2 -column 0
grid .f1.keepMe.save -row 3 -column 0

##########
# Arrange the frames in the GUI window
#
grid .f1 -row 0 -sticky ew

###
# Resets the camera. 
#
proc resetCamera {args} {
	ren1 ResetCamera
	ren1 ResetCameraClippingRange
	[ren1 GetActiveCamera] OrthogonalizeViewUp
	[ren1 GetActiveCamera] SetViewUp 0 1 0
	renWin Render
}