# Zach Gildersleeve
# zgildersleeve@gmail.com
# CS6630 - Scientific Visualization
# Project 4
# November 22, 2006
#
# volume.tcl - This script builds a composite based volume
# rendered image, using a custom defined transfer function.
# The volume dataset is cast into a vtkVolume object.
# Multiple options exist under the vtkStructuredPointsReader,
# vtkVolumePropery, and vtkVolumeRayCastMapper that allow
# for experimentation of the resolution, interpolation, and
# sample distance parameters, respectively. No GUI control over
# these parameters is provided, as the results of experimentation
# can be seen in the report.

catch {load vtktcl}
set VTK_TCL ".."
set VTK_DATA "../vtkdata"

##########
# Read in the data sets
#
vtkStructuredPointsReader reader_mummy
#	reader_mummy SetFileName $VTK_DATA/mummy.50.vtk
#	reader_mummy SetFileName $VTK_DATA/mummy.80.vtk
	reader_mummy SetFileName $VTK_DATA/mummy.128.vtk
	reader_mummy Update

##########
# Build the different transfer function components
#
vtkPiecewiseFunction opacityTransferFunction_mummy
	opacityTransferFunction_mummy AddPoint	00	0.0
	opacityTransferFunction_mummy AddPoint	60	0.0
	opacityTransferFunction_mummy AddPoint	70	0.05
	opacityTransferFunction_mummy AddPoint	80	0.2
	opacityTransferFunction_mummy AddPoint	95	0.2
	opacityTransferFunction_mummy AddPoint	100	0.0
	opacityTransferFunction_mummy AddPoint	110	0.0
	opacityTransferFunction_mummy AddPoint	115	0.6
	opacityTransferFunction_mummy AddPoint	255	0.6

vtkColorTransferFunction colorTransferFunction_mummy
	colorTransferFunction_mummy AddRGBPoint	      0.0	0.0	0.0	0.0
	colorTransferFunction_mummy AddRGBPoint      60.0	0.8	0.2	0.2
	colorTransferFunction_mummy AddRGBPoint     100.0	0.8	0.2	0.2
	colorTransferFunction_mummy AddRGBPoint     110.0	0.9	0.9	1.0

##########
# Create the vtkVolume via a vtkVolumeRayCastMapper mapping function
#
vtkVolumeProperty volumeProperty
	volumeProperty SetColor colorTransferFunction_mummy
	volumeProperty SetScalarOpacity opacityTransferFunction_mummy
	volumeProperty ShadeOn
	volumeProperty SetSpecular 0.3
	volumeProperty SetInterpolationTypeToLinear
#	volumeProperty SetInterpolationTypeToNearest

vtkVolumeRayCastCompositeFunction compositeFunction

vtkVolumeRayCastMapper volumeMapper
#	volumeMapper SetSampleDistance 0.0625
#	volumeMapper SetSampleDistance 0.125
#	volumeMapper SetSampleDistance 0.25
#	volumeMapper SetSampleDistance 0.5
	volumeMapper SetSampleDistance 1.0
#	volumeMapper SetSampleDistance 2.0
#	volumeMapper SetSampleDistance 4.0
#	volumeMapper SetSampleDistance 8.0
#	volumeMapper SetSampleDistance 16.0
	volumeMapper SetInput [reader_mummy GetOutput]
	volumeMapper SetVolumeRayCastFunction compositeFunction

vtkVolume volume
	volume SetMapper volumeMapper
	volume SetProperty volumeProperty
	volume RotateX -90

##########
# Create the Renderer and Windows
#
vtkRenderer ren1
	
vtkRenderWindow renWin
	renWin AddRenderer ren1
	renWin SetSize 512 512
vtkRenderWindowInteractor iren
	iren SetRenderWindow renWin
	ren1 AddVolume volume
	ren1 SetBackground 0.5 0.5 0.5
	ren1 ResetCamera

vtkInteractorStyleTrackballCamera style
	iren SetInteractorStyle style

##########
# Adapted from camframe.tcl
#
proc saveView {} {
  set chan [open "saveView.tcl" w]
  set cam [ren1 GetActiveCamera]
  puts $chan "set cam \[ren1 GetActiveCamera\]"
  puts $chan [format "\$cam SetPosition %s" [$cam GetPosition]]
  puts $chan [format "\$cam SetViewUp %s" [$cam GetViewUp]]
  puts $chan [format "\$cam SetFocalPoint %s" [$cam GetFocalPoint]]
  puts $chan [format "\$cam SetViewAngle %s" [$cam GetViewAngle]]
  puts $chan [format "renWin SetSize %s" [renWin GetSize]]
  puts $chan "ren1 ResetCameraClippingRange"
  close $chan
}

# if a previous run of this program generated the camera information file,
# then read it in now, before the first rendering
if {[file exists "saveView.tcl"]} {
    source "saveView.tcl"
}

renWin Render

#########
# Set up the GUI widgets
#
frame .f1
frame .f1.keepMe
button .f1.keepMe.quit -text "Quit" -command {exit}
button .f1.keepMe.reset -text "Reset Camera" -command resetCamera
button .f1.keepMe.save -text "save view" -command saveView

##########
# Arrange the widgets and GUI elements in each
# respective frame
#
grid .f1.keepMe -row 0 -column 0
grid .f1.keepMe.quit -row 1 -column 0
grid .f1.keepMe.reset -row 2 -column 0
grid .f1.keepMe.save -row 3 -column 0

##########
# Arrange the frames in the GUI window
#
grid .f1 -row 0 -sticky ew

###
# Resets the camera. 
#
proc resetCamera {args} {
	ren1 ResetCamera
	ren1 ResetCameraClippingRange
	[ren1 GetActiveCamera] OrthogonalizeViewUp
	[ren1 GetActiveCamera] SetViewUp 0 1 0
	renWin Render
}